#!/usr/bin/env python3

##########################
#     WRAPPER SCRIPT     #
#     DEALS WITH MISP    #
#     TO STIX            #
##########################


import argparse
import pyaml
import sys
import os

from misp_stix_converter.servers import misp
from misp_stix_converter.converters import convert
from misp_stix_converter.converters import lint_roller

parser = argparse.ArgumentParser(description='Process some integers.')
parser.add_argument('-o', '--outfile', help="The file to output to. Default is stdout")
parser.add_argument("-c", "--config", help="Path to config file. Default is misp.login.")
parser.add_argument("-f", "--file", help="The MISP JSON file to convert")
parser.add_argument("-i", "--eid", help="The MISP event ID to pull and convert")
parser.add_argument("--format", help="The output format [JSON/XML]. Default JSON.")
parser.add_argument("--stix-version", help="Set the STIX output version. Default 1.2.")

args = parser.parse_args()

# Set the config file
if args.config:
    configfile = args.config
else:
    configfile = os.path.expanduser("~/.misptostix/misp.login")

try:
    with open(configfile, "r") as f:
        CONFIG = pyaml.yaml.load(f)
except FileNotFoundError:
    print("Could not find config file {}".format(configfile))
    sys.exit(1)

# We either need a file or an event ID
if not (args.file or args.eid):
    print("We need something to convert!")
    print("Please either specify a file with -f [FILENAME]")
    print("Or a MISP ID with -i [ID]")
    sys.exit()

if (args.file and args.eid):
    print("We can't convert both at once!")
    print("*EITHER* provide -i or -f. Not both.")
    sys.exit()

if args.format:
    args.format = args.format.lower()
    if args.format not in ["json", "xml"]:
        print("Only possible output formats are JSON and XML.")
        print("{} is not valid".format(args.format))
        sys.exit()
else:
    args.format = "json"

if (args.file):
    # This is just a file conversion
    # Relatively quick and easy
    # Create a non-connected misp instance
    try:
        with open(args.file, "r") as f:
            jsondata = f.read()
        package = convert.MISPtoSTIX(jsondata)
    except FileNotFoundError:
        print("Could not open {}".format(args.file))
        sys.exit()

else:
    # This requires a connection to MISP
    # As we need to pull an event
    # Connect to MISP
    MISP = misp.MISP(CONFIG["MISP"]["URL"], CONFIG["MISP"]["KEY"])
    package = MISP.pull(args.eid)[0]


# Set the version
if args.stix_version:
    if args.stix_version == "1.1.1":
        objs = lint_roller.lintRoll(package)
        for i in objs:
            # Set the object's version
            if hasattr(i, "version"):
                i.version = args.stix_version

    elif args.stix_version == "1.2":
        pass  # Is default
    else:
        print("INVALID STIX VERSION {}".format(args.stix_version))
        sys.exit()

if args.format == "json":
    # Output to JSON
    if not args.outfile:
        # Output to stdout
        print(package.to_json())
    else:
        # Output to file
        with open(args.outfile, "w") as f:
            f.write(package.to_json())
else:
    # Output to XML
    if not args.outfile:
        # Output to stdout
        print(package.to_xml())
    else:
        # Output to file
        with open(args.outfile, "wb") as f:
            f.write(package.to_xml())



#!/usr/bin/env python3

# A method to get allllll objects out of a STIX package.


def lintRoll(pkg):
    objs = []
    if hasattr(pkg, "walk"):
        # This means we can get more objects out of it
        for x in pkg.walk():
            #print(type(x), x)
            objs += lintRoll(x)
    else:
        # Make sure it'll return itself
        objs.append(pkg)
    return objs



#!/usr/bin/env python3
import re
import cybox
import logging
import hashlib
import six

from pymisp import mispevent

from misp_stix_converter.converters.lint_roller import lintRoll

from stix.core import STIXPackage

# Cybox cybox don't we all love cybox children
from cybox.objects import email_message_object, file_object, address_object
from cybox.objects import domain_name_object, hostname_object, uri_object
from cybox.objects import mutex_object, whois_object
from cybox.objects import as_object, http_session_object
from cybox.objects import pipe_object, network_packet_object, win_registry_key_object


# Just a little containment file for STIX -> MISP conversion
ipre = re.compile("([0-9]{1,3}.){3}[0-9]{1,3}")
log = logging.getLogger("__main__")


def identifyHash(hsh):
    """
        What's that hash!?
    """

    possible_hashes = []

    hashes = [x for x in hashlib.algorithms_guaranteed]

    for h in hashes:
        if len(str(hsh)) == len(hashlib.new(h).hexdigest()):
            possible_hashes.append(h)
            possible_hashes.append("filename|{}".format(h))
    return possible_hashes


def open_stix(stix_thing):
    # Load the package
    if not hasattr(stix_thing, 'read'):
        stix_thing = open(stix_thing, "r")

    pkg = None
    try:
        pkg = STIXPackage().from_xml(stix_thing)
    except:
        try:
            pkg = STIXPackage.from_json(stix_thing)
        except:
            raise Exception("Could not load package!")
    return pkg


def buildEvent(pkg, **kwargs):
    log.info("Building Event...")
    if not pkg.stix_header:
        title = "STIX Import"
    else:
        if not pkg.stix_header.title:
            title = "STIX Import"
        else:
            title = pkg.stix_header.title
    log.info(title)
    event = mispevent.MISPEvent()
    event.distribution = kwargs.get("distribution", 0)
    event.threat_level_id = kwargs.get("threat_level_id", 3)
    event.analysis = kwargs.get("analysis", 0)
    event.info = title

    ids = []
    to_process = []
    for obj in lintRoll(pkg):
        if isinstance(obj, cybox.core.observable.Observable):
            if obj.id_ not in ids:
                ids.append(obj.id_)
                to_process.append(obj)

    for obj in to_process:
        # This will find literally every object ever.
        event = buildAttribute(obj, event)
    return event


def buildAttribute(pkg, mispEvent):
    try:
        # Check if the object is a cybox observable
        if isinstance(pkg, cybox.core.observable.Observable):
            if hasattr(pkg, "object_") and pkg.object_:
                obj = pkg.object_.properties
                # It's a proper object!
                type_ = type(obj)
                # Here comes the fun!
                if type_ == address_object.Address:
                    # We've got an address object, naturally
                    # We can check if it's a source or dest
                    if obj.is_source:
                        mispEvent.add_attribute('ip-src', six.text_type(obj.address_value),
                                                comment=pkg.title or "")
                    elif obj.is_destination:
                        mispEvent.add_attribute('ip-dst', six.text_type(obj.address_value),
                                                comment=pkg.title or "")
                    else:
                        # Don't have anything to go on
                        mispEvent.add_attribute('ip-dst', six.text_type(obj.address_value),
                                                comment=pkg.title or "")
                elif type_ == domain_name_object.DomainName:
                    mispEvent.add_attribute('domain', six.text_type(obj.value), comment=pkg.title or "")
                elif type_ == hostname_object.Hostname:
                    mispEvent.add_attribute('hostname', six.text_type(obj.hostname_value),
                                            comment=pkg.title or "")
                elif type_ == uri_object.URI:
                    mispEvent.add_attribute('url', six.text_type(obj.value),
                                            comment=pkg.title or "")
                elif type_ == file_object.File:
                    # This is a bit harder
                    # TODO: This
                    pass

                elif type_ == email_message_object.EmailMessage:
                    if obj.header:
                        # We have a header, can check for to/from etc etc
                        if obj.header.from_:
                            mispEvent.add_attribute('email-src', six.text_type(obj.header.from_.address_value),
                                                    comment=pkg.title or "")
                        if obj.header.to:
                            for mail in obj.header.to:
                                mispEvent.add_attribute('email-dst', six.text_type(mail.address_value),
                                                        comment=pkg.title or "")
                        if obj.header.subject:
                            mispEvent.add_attribute('email-subject', six.text_type(obj.header.subject),
                                                    comment=pkg.title or "")
                    if obj.attachments:
                        # FIXME that's definitely broken, but I have no sample.
                        for att in obj.attachments:
                            mispEvent.add_attribute('email-attachment', att.value,
                                                    comment=pkg.title or "")
                elif type_ == mutex_object.Mutex:
                    mispEvent.add_attribute('mutex', obj.name, comment=pkg.title or "")
                elif type_ == whois_object.WhoisEntry:
                    pass
                elif type_ == win_registry_key_object.WinRegistryKey:
                    pass
                elif type_ == network_packet_object.NetworkPacket:
                    pass
                elif type_ == http_session_object.HTTPSession:
                    pass
                elif type_ == pipe_object.Pipe:
                    mispEvent.add_attribute('named pipe', six.text_type(obj.name), comment=pkg.title or "")
                elif type_ == as_object.AS:
                    mispEvent.add_attribute('AS', six.text_type(obj.number), comment=pkg.title or six.text_type(obj.name) or "")
                else:
                    log.debug("Type not syncing {}".format(type_))
            else:
                pass
        else:
            pass  # Other objects. TODO.
    except Exception as ex:
        log.error(ex)
    return mispEvent



#!/usr/bin/env python3

# A decoupled converter for misp<->Stix

# Imports
# Sys imports
import logging
from tempfile import NamedTemporaryFile
import json

from pymisp import mispevent

# Stix imports
from stix.core import STIXPackage
from stix.core import STIXHeader
from stix.indicator import Indicator

# Local imports
from misp_stix_converter.errors import STIXLoadError
from misp_stix_converter.converters import buildSTIXAttribute
from misp_stix_converter.converters import buildMISPAttribute

log = logging.getLogger("__main__")


def MISPtoSTIX(mispJSON):
    """
        Function to convert from a MISP JSON to a STIX stix

        :param mispJSON: A dict (json) containing a misp Event.
        :returns stix: A STIX stix with as much of the original
                          data as we could convert.
    """
    if isinstance(mispJSON, mispevent.MISPEvent):
        misp_event = mispJSON
    else:
        misp_event = mispevent.MISPEvent()
        misp_event.load(mispJSON)

    # We should now have a proper MISP JSON loaded.

    # Create a base stix
    stix = STIXPackage()

    # Create a header for the new stix
    stix.stix_header = STIXHeader()

    # Try to use the event title as the stix title
    stix.stix_header.title = misp_event.info

    # We're going to store our observables inside an indicator
    indicator = Indicator()

    # Go through each attribute and transfer what we can.
    for one_attrib in misp_event.attributes:
        # Build an attribute from the JSON. Is all nice.
        buildSTIXAttribute.buildAttribute(one_attrib, stix, indicator)
    stix.add_indicator(indicator)
    return stix


def load_stix(stix):
    # Just save the pain and load it if the first character is a <

    if isinstance(stix, STIXPackage):
        return stix

    f = NamedTemporaryFile(mode="w+")
    f.write(stix)
    f.seek(0)
    # Oh no we have to try and load it now
    try:
        # Try loading from JSON
        stix_package = STIXPackage().from_json(f.name)
    except:
        # Ok then try loading from XML
        try:
            stix_package = STIXPackage().from_xml(f.name)
        except Exception as ex:
            # No joy. Quit.
            raise STIXLoadError("Could not load stix file. {}".format(ex))
    return stix_package


def STIXtoMISP(stix, mispAPI, **kwargs):
    """
        Function to convert from something stixxy ( as we have 3 possible representations )
        to something mispy. Specifically JSON. Because XML is satan.

        :param stix: Something stixxy.
    """

    log.info("Converting a package from STIX to MISP...")
    stix = load_stix(stix)
    # Ok by now we should have a proper STIX object.

    misp_event = buildMISPAttribute.buildEvent(stix, **kwargs)
    if misp_event.attributes:
        response = mispAPI.add_event(json.dumps(misp_event, cls=mispevent.EncodeUpdate))
        if response.get('errors'):
            # FIXME *maybe* we want to raise a thing there....
            pass
            # raise Exception(response.get('errors'))

        return response



#!/usr/bin/env python3

# A file to store the buildAttribute() function
# as it goes on for quite a while.

# It's... well, it's a containment file.

# Cybox is awful
import stix
from stix.extensions.test_mechanism import snort_test_mechanism, yara_test_mechanism

# No you can't go
# from cybox.objects import *
# Because cybox is terrible.
from cybox.objects import email_message_object, file_object, address_object
from cybox.objects import domain_name_object, hostname_object, uri_object
from cybox.objects import link_object, mutex_object, whois_object
from cybox.objects import x509_certificate_object, as_object, http_session_object
from cybox.objects import pipe_object, network_packet_object, win_registry_key_object
from cybox.common.hashes import Hash

import logging
log = logging.getLogger("__main__")


def buildAttribute(attr, pkg, ind):
    """
      Given a MISP attribute, create a stix
      attribute, add it to an indicator for
      easy lookings up.

      There are quite a few assumptions here,
      including that there should only be one
      threat-actor per MISP event in order to
      give proper attribution.

      :param attr: The misp JSON attribute to parse and build from
      :param pkg : The STIX package to push the created object to
      :param ind : The STIX indicator to push observables to
   """

    # Extract
    type_ = attr.type
    value = attr.value

    if type_ == "ip-src":
        # An IP address. Add it as an Address Object.
        addr = address_object.Address(address_value=value)
        addr.is_source = True
        addr.is_destination = False
        obs = stix.indicator.Observable(addr)
        obs.title = attr.comment or "IP Source"
        ind.add_observable(obs)

    elif type_ == "ip-dst":
        # An IP address. Add it as an Address Object.
        addr = address_object.Address(address_value=value)
        addr.is_source = False
        addr.is_destination = True
        obs = stix.indicator.Observable(addr)
        obs.title = attr.comment or "IP Destination"
        ind.add_observable(obs)

    elif type_ == "domain":
        # A domain. Add as a DomainName Object.
        dn = domain_name_object.DomainName()
        dn.value = value
        obs = stix.indicator.Observable(dn)
        obs.title = attr.comment or "Domain"
        ind.add_observable(obs)

    elif type_ == "hostname":
        # A hostname. Add as Hostname Object.
        hst = hostname_object.Hostname()
        hst.hostname_value = value
        obs = stix.indicator.Observable(hst)
        obs.title = attr.comment or "Hostname"
        ind.add_observable(obs)

    elif type_ in ["url", "uri"]:
        # UR(i|l). I guess we'll use a URI object (as URLs ⊆ URIs).
        url = uri_object.URI(value)
        obs = stix.indicator.Observable(url)
        obs.title = attr.comment or "URI"
        ind.add_observable(obs)

    elif type_ in ["md5", "sha1", "sha256", "sha512"]:
        # A definite hash. They all come under SimpleHashValue.
        hsh = Hash(value)
        f = file_object.File()
        if type_ == "md5":
            f.md5 = value
        elif type_ == "sha1":
            f.sha1 = value
        elif type_ == "sha256":
            f.sha256 = value
        elif type_ == "sha512":
            f.sha512 == value
        obs = stix.indicator.Observable(f)
        obs.title = attr.comment or "Hash (Simple)"
        ind.add_observable(f)

    elif type_ == "filename":
        # Just a filename. Add it to a File Object, then add that.
        f = file_object.File()
        f.file_name = value
        obs = stix.indicator.Observable(f)
        obs.title = attr.comment or "Filename"
        ind.add_observable(f)

    elif type_ in ["filename|md5", "filename|sha1", "filename|sha256", "filename|sha512"]:
        # A filename AND a hash! Aren't we lucky!
        # Add the Hash to a File object.
        fname, sep, hsh = value.partition("|")
        f = file_object.File()
        if "md5" in type_:
            f.md5 = hsh
        elif "sha1" in type_:
            f.sha1 = hsh
        elif "sha256" in type_:
            f.sha256 = hsh
        elif "sha512" in type_:
            f.sha512 = hsh
        f.file_name = fname
        obs = stix.indicator.Observable(f)
        obs.title = attr.comment or "File"
        ind.add_observable(f)

    elif type_ in ["ssdeep", "authentihash", "imphash"]:
        # A fuzzy hash. They're a bit weirder, but
        # we'll handle them nonetheless.
        hsh = Hash()
        f = file_object.File()
        hsh.fuzzy_hash_value = value
        f.add_hash(hsh)
        obs = stix.indicator.Observable(f)
        obs.title = attr.comment or "Hash (Fuzzy)"
        ind.add_observable(f)

    elif type_ == "threat-actor":
        # Threat Actors. Whilst we don't have proper attribution,
        # we can still add them as a thing.
        ta = stix.common.ThreatActor()
        ta.title = value
        if "comment" in attr:
            ta.description = attr.comment
        pkg.add_threat_actor(ta)

    elif type_ == "campaign-name":
        # Just a campaign name. Is nice.
        # Would be nice if we could structure it
        # so that it went Camp -> Obs, but sadly
        # we can't rely on a campaign existing.
        camp = stix.common.Campaign(title=value)
        pkg.add_campaign(camp)

    elif type_ == "link":
        # Arbritary link value.
        lnk = link_object.Link(value)
        obs = stix.indicator.Observable(lnk)
        obs.title = attr.comment or "Link"
        ind.add_observable(obs)

    elif type_ == "email-src":
        # Email Source. Create an Email Object,
        # then add the address in the header.
        emsg = email_message_object.EmailMessage()
        esrc = email_message_object.EmailHeader()
        esrc.from_ = value
        emsg.header = esrc
        obs = stix.indicator.Observable(emsg)
        obs.title = attr.comment or "Email Source Address"
        ind.add_observable(obs)

    elif type_ == "email-subject":
        # Same as above, but a subject. Add it
        # to the header.
        emsg = email_message_object.EmailMessage()
        esub = email_message_object.EmailHeader()
        esub.subject = value
        emsg.header = esub
        obs = stix.indicator.Observable(emsg)
        obs.title = attr.comment or "Email Subject Line"
        ind.add_observable(obs)

    elif type_ == "email-attachment":
        # Filename of an attachment.
        emsg = email_message_object.EmailMessage()
        att = email_message_object.Attachments()
        att.append(value)
        emsg.attachments = att
        obs = stix.indicator.Observable(emsg)
        obs.title = attr.comment or "Email Attachment"
        ind.add_observable(obs)

    elif type_ in ["email-dst", "target-email"]:
        # The "TO" field of an email address
        # Easy enough, just create an email object and shove it
        # in the hdr.
        emsg = email_message_object.EmailMessage()
        esub = email_message_object.EmailHeader()
        esub.to = value
        emsg.header = esub
        obs = stix.indicator.Observable(emsg)
        obs.title = attr.comment or "Email Destination Address"
        ind.add_observable(obs)

    elif type_ == "attachment":
        # This one is debatable.
        # I'll ignore it for now.
        pass

    elif type_ == "mutex":
        # A malware Mutex.
        # Just an fancy observable.
        mut = mutex_object.Mutex()
        mut.name = value
        obs = stix.indicator.Observable(mut)
        obs.title = attr.comment or "Mutex"
        ind.add_observable(obs)

    elif type_ == "x509-fingerprint-sha1":
        # Not directly transferrable. Best we can do
        # it putting it on the signature.
        cert = x509_certificate_object.X509CertificateSignature()
        cert.signature = "FINGERPRINT: {}".format(value)
        # TODO: Figure out how to add this to a package

    elif type_ == "whois-registrant-email":
        # This is far too much work for an email
        # Goddamnit STIX.
        # Create a whois entry
        whois = whois_object.WhoisEntry()
        # And a list of registrants
        # Which is apparently its own object for some reason
        regs = whois_object.WhoisRegistrants()
        reg = whois_object.WhoisRegistrant()
        # And add the email address
        reg.email_address = value
        regs.append(reg)
        whois.registrants = regs
        obs = stix.indicator.Observable(whois)
        obs.title = attr.comment or "WHOIS Email"
        ind.add_observable(obs)

    elif type_ == "whois-registrant-name":
        # I swear stix just LOVES to add layers
        # Just a name of the registrant
        whois = whois_object.WhoisEntry()
        regs = whois_object.WhoisRegistrants()
        reg = whois_object.WhoisRegistrant()
        reg.name = value
        regs.append(reg)
        whois.registrants = regs
        obs = stix.indicator.Observable(whois)
        obs.title = attr.comment or "WHOIS Registrant Name"
        ind.add_observable(obs)

    elif type_ == "whois-creation-date":
        # And the date the whois was created.
        # Nothing out of the ordinary
        whois = whois_object.WhoisEntry()
        whois.creation_date = value
        obs = stix.indicator.Observable(whois)
        obs.title = attr.comment or "WHOIS Creation Date"
        ind.add_observable(obs)

    elif type_ == "whois-registrar":
        # Aaaand the registrar
        whois = whois_object.WhoisEntry()
        reg = whois_object.WhoisRegistrar()
        reg.name = value
        whois.registrar_info = reg
        obs = stix.indicator.Observable(whois)
        obs.title = attr.comment or "WHOIS Registrar"
        ind.add_observable(obs)

    elif type_ == "pdb":
        # NOT SUPPORTED
        pass

    elif type_ == "domain|ip":
        # Oooh we've got both!
        # We'll add them in turn
        dom, sep, ip = value.partition("|")

        # Add the IP
        addr = address_object.Address(address_value=ip)
        obs = stix.indicator.Observable(addr, title=attr.comment)
        obs.title = attr.comment or "IP Address"
        ind.add_observable(obs)

        # Now add the domain
        dn = domain_name_object.DomainName()
        dn.value = dom
        obs = stix.indicator.Observable(dn)
        obs.title = attr.comment or "Domain"
        ind.add_observable(obs)

    elif type_ == "vulnerability":
        # It's a CVE. Easy enough to deal with.
        vuln = stix.exploit_target.Vulnerability()
        vuln.cve_id = value
        et = stix.exploit_target.ExploitTarget()
        et.title = attr.comment or "Vulnerability"
        et.add_vulnerability(vuln)
        pkg.add_exploit_target(et)

    elif type_ == "snort":
        # Some snort rule. Idk what Snort is or does,
        # but still, we'll take it.
        snort = snort_test_mechanism.SnortTestMechanism()
        snort.add_rule(value)
        ind.test_mechanisms.append(snort)

    elif type_ == "yara":
        # Now this I know. YARA test rules.
        # Add it to the list of test mechanisms.
        yara = yara_test_mechanism.YaraTestMechanism()
        yara.rule = value
        ind.test_mechanisms.append(yara)

    elif type_ == "regkey|value":
        # A windows registry key and a value
        # Split them and add seperately
        regkey, sep, value = value.partition("|")
        regentry = win_registry_key_object.WinRegistryKey()
        val = win_registry_key_object.RegistryValue()
        val.name = regkey
        val.data = value
        vals = win_registry_key_object.RegistryValues()
        vals.append(val)
        regentry.values = vals
        obs = stix.indicator.Observable(regentry)
        ind.add_observable(obs)

    elif type_ == "regkey":
        # Just a reg key without a value.
        regkey, sep, value = value.partition("|")
        regentry = win_registry_key_object.WinRegistryKey()
        val = win_registry_key_object.RegistryValue()
        val.name = regkey
        vals = win_registry_key_object.RegistryValues()
        vals.append(val)
        regentry.values = vals
        obs = stix.indicator.Observable(regentry)
        obs.title = attr.comment or "Registry Key"
        ind.add_observable(obs)

    elif type_ == "pattern-in-traffic":
        # Create a packet and set the pattern as the data
        pack = network_packet_object.IPv4Packet()
        pack.data = value
        lay = network_packet_object.InternetLayer()
        lay.ipv4 = pack
        net = network_packet_object.NetworkPacket()
        net.internet_layer = lay
        obs = stix.indicator.Observable(net)
        obs.title = attr.comment or "Pattern In Traffic"
        ind.add_observable(obs)

    elif type_ == "user-agent":
        # Probably in the hdr of a HTTP req
        # My god this goes on for a while
        http = http_session_object.HTTPRequestHeaderFields()
        http.user_agent = value
        hdr = http_session_object.HTTPRequestHeader()
        hdr.parsed_header = http
        req = http_session_object.HTTPClientRequest()
        req.http_request_header = hdr
        resp = http_session_object.HTTPRequestResponse()
        resp.http_client_request = req
        ses = http_session_object.HTTPSession()
        ses.http_request_response = resp
        obs = stix.indicator.Observable(ses)
        obs.title = attr.comment or "User Agent"
        ind.add_observable(obs)

    elif type_ == "named pipe":
        # Pipe pipe pipe pipe pipe pipe
        p = pipe_object.Pipe()
        p.name = value
        obs = stix.indicator.Observable(p)
        obs.title = attr.comment or "Named Pipe"
        ind.add_observable(obs)

    elif type_ == "AS":
        as_ = as_object.AS()
        try:
            as_.number = value
        except ValueError:
            as_.name = value
        obs = stix.indicator.Observable(as_)
        obs.title = attr.comment or "Autonomous System"
        ind.add_observable(obs)

    else:
        # Known unsupported
        if type_ not in ["campaign-id", "comment", "text",
                         "malware-sample", "pattern-in-file",
                         "other"]:
            log.debug("Not adding {}".format(type_))



#!/usr/bin/env python3

#############################
#           MISP            #
#    Wrapper to PyMISP      #
#                           #
#   Provides push/pull      #
#############################

# A load of random imports
# Very very useful
import requests
import pymisp
import logging
from misp_stix_converter.converters import convert

__author__ = "Hannah Ward"

log = logging.getLogger("__main__")


class MISP:
    """
      Wrapper to MISP API -- allows easy exporting and
      importing of STIX data
    """

    def __init__(self, url, key, verify=False, misp_modules=False):
        """
          Initialise the MISP instance

          :param url: The URL of the MISP instance
          :param key: The API key to authenticate with
          :param verify: Shall we verify SSL? Can be bool or location of a signature
        """
        log.debug("Starting MISP")
        log.debug("{} -- {}".format(url, key))
        self.url = url
        self.key = key
        self.verify = verify
        self.mispAPI = pymisp.PyMISP(url, key, ssl=self.verify, debug=False)
        self.misp_modules = misp_modules

    def pull(self, id_=None, tags=None, from_=None, to=None):
        """
          Pull the requested events from the MISP server

          :param id_: Only export events with matching IDs
          :param tags: Only export events matching these tags
          :param from_: Only export events created after this date (YYYY-MM-DD)
          :param to: Only export events created before this date (YYYY-MM-DD)
        """

        log.debug("MISP PULL")
        log.debug("IDS:  {}".format(id_))
        log.debug("TAGS: {}".format(tags))
        log.debug("FROM: {}".format(from_))
        log.debug("TO:   {}".format(to))

        # Attempt to request the data
        try:
            log.info("Sending now...")

            recent = self.mispAPI.search_index(eventid=id_,
                                               tag=tags,
                                               datefrom=from_,
                                               dateto=to,
                                               )

        except requests.exceptions.HTTPError as ex:
            # 500 Internal -- Usually when there's no results
            log.error("MISP returned an error")
            log.error(ex)
            # Just send back nothing
            return []

        except AttributeError as ex:
            log.warning(ex)
            log.error("Server error - no data recieved")
        log.info("Response recieved, MISP pull complete.")

        log.info("{} packages recieved".format(len(recent["response"])))
        packages = [convert.MISPtoSTIX(self.mispAPI.get(x["id"])) for x in recent["response"]]
        return packages

    def push(self, data, dryrun=False, **kwargs):
        """
          Push a package to the MISP instance

          :param data: The STIX package to push
          :param distribution: The dist number of the event [0,1,2,3]
          :param threat_level_id: How much of a threat is posed? [1-4, 1 highest]
          :param analysis: How far are we through analysis? [0-3, 3 highest]
          :param tags: The tags to assign the new event.
          :param verified: Is the event verified? Default False.
        """

        # If we've been given a list, push each in turn
        if isinstance(data, list):
            for i in data:
                self.push(i)

        else:
            convert.STIXtoMISP(data, self.mispAPI, **kwargs)
        return True



#!/usr/bin/env python3

# Setup script for ThreatIntel Conversion

from setuptools import setup
import os

setup(
    name="misp_stix_converter",
    description="A tool to convert between STIX and MISP format",
    version="0.2",
    author="Hannah Ward",
    author_email="hannah.ward@baesystems.com",
    url="https://github.com/FloatingGhost/MISP-STIX-Converter",
    packages=['misp_stix_converter', "misp_stix_converter.servers", "misp_stix_converter.converters"],
    install_requires=["pymisp>=2.4.53", "requests>=2.9.1", "pyaml>=3.11",
                      "stix>=1.2", "cybox>=2.0", "nose", "cabby"],
    dependency_links=["https://github.com/MISP/PyMISP.git",
                      "https://github.com/STIXProject/python-stix.git",
                      "https://github.com/CybOXProject/python-cybox.git"],
    scripts=["misp_stix_converter/misp-to-stix.py", "misp_stix_converter/stix-to-misp.py"],
    data_files=[(os.path.expanduser("~/.misptostix"),
                ['misp.login.example'])]
)



#!/usr/bin/env python3


class MISPLoadError(Exception):
    pass


class STIXLoadError(Exception):
    pass


class STIXConversionError(STIXLoadError):
    pass


class MISPConversionError(MISPLoadError):
    pass



#!/usr/bin/env python3

##########################
#     WRAPPER SCRIPT     #
#     DEALS WITH STIX    #
#     TO MISP            #
##########################


import argparse
import pyaml
import sys
import os

from misp_stix_converter.servers import misp
from misp_stix_converter.converters.buildMISPAttribute import open_stix

parser = argparse.ArgumentParser(description='Process some integers.')
parser.add_argument("-c", "--config", help="Path to config file. Default is misp.login.")
parser.add_argument("file", help="The STIX file to push")

args = parser.parse_args()

# Set the config file
if args.config:
    configfile = args.config
else:
    configfile = os.path.expanduser("~/.misptostix/misp.login")

try:
    with open(configfile, "r") as f:
        CONFIG = pyaml.yaml.load(f)
except FileNotFoundError:
    print("Could not find config file {}".format(configfile))
    sys.exit(1)

# This is just a file conversion
# Relatively quick and easy
MISP = misp.MISP(CONFIG["MISP"]["URL"], CONFIG["MISP"]["KEY"])

# Load the package
pkg = open_stix(args.file)

# We'll use my nice little misp module
MISP.push(pkg)
